@@ -41,7 +41,7 @@
 
 // These number should fit for most formats; see comments in the alffmpeg.c example.
 const size_t audio_output::_num_buffers = 3;
-const size_t audio_output::_buffer_size = 20160;
+const size_t audio_output::_buffer_size = 20160 * 2;
 
 audio_output::audio_output(bool receive_notifications) :
     controller(receive_notifications),
