@@ -28,6 +28,11 @@
 #include <limits>
 #include <sstream>
 
+#include <iconv.h>
+
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "dbg.h"
 #include "msg.h"
 
@@ -194,14 +199,14 @@ namespace str
     /* Convert a string to one of the basic data types */
 
     template<typename T>
-    static inline T _to(const std::string &s, const std::string &name)
+    static inline T _to(const std::string &s, const char *name)
     {
         std::istringstream is(s);
         T v;
         is >> v;
         if (is.fail() || !is.eof())
         {
-            throw exc(std::string("Cannot convert '") + sanitize(s) + "' to " + name, EINVAL);
+            throw exc(str::asprintf(_("Cannot convert string to %s."), name), EINVAL);
         }
         return v;
     }
@@ -357,4 +362,61 @@ namespace str
             return str::asprintf("%.1f mm", length * 1000.0);
         }
     }
+
+    /* Convert a string from one character set to another */
+
+    std::string convert(const std::string &src, const std::string &from_charset, const std::string &to_charset)
+    {
+        if (from_charset.compare(to_charset) == 0)
+        {
+            return src;
+        }
+
+        iconv_t cd = iconv_open(to_charset.c_str(), from_charset.c_str());
+        if (cd == reinterpret_cast<iconv_t>(static_cast<size_t>(-1)))
+        {
+            throw exc(str::asprintf(_("Cannot convert %s to %s."), from_charset.c_str(), to_charset.c_str()), errno);
+        }
+
+        size_t inbytesleft = src.length() + 1;
+        const char *inbuf = src.c_str();
+        size_t outbytesleft = inbytesleft;
+        if (outbytesleft >= std::numeric_limits<size_t>::max() / 4)
+        {
+            outbytesleft = std::numeric_limits<size_t>::max();
+        }
+        else
+        {
+            outbytesleft = outbytesleft * 4;
+        }
+        char *orig_outbuf = static_cast<char *>(malloc(outbytesleft));
+        if (!orig_outbuf)
+        {
+            iconv_close(cd);
+            throw exc(str::asprintf(_("Cannot convert %s to %s."), from_charset.c_str(), to_charset.c_str()), ENOMEM);
+        }
+        char *outbuf = orig_outbuf;
+
+        size_t s = iconv(cd, const_cast<char **>(&inbuf), &inbytesleft, &outbuf, &outbytesleft);
+        int saved_errno = errno;
+        iconv_close(cd);
+        if (s == static_cast<size_t>(-1))
+        {
+            free(orig_outbuf);
+            throw exc(str::asprintf(_("Cannot convert %s to %s."), from_charset.c_str(), to_charset.c_str()), saved_errno);
+        }
+
+        std::string dst;
+        try
+        {
+            dst.assign(orig_outbuf);
+        }
+        catch (std::exception &e)
+        {
+            free(orig_outbuf);
+            throw exc(str::asprintf(_("Cannot convert %s to %s."), from_charset.c_str(), to_charset.c_str()), ENOMEM);
+        }
+        free(orig_outbuf);
+        return dst;
+    }
 }
