@@ -83,7 +83,9 @@ public:
         adjust_subtitle_parallax,       // float (relative adjustment)
         set_subtitle_parallax,          // float (absolute value)
         seek,                           // float (relative adjustment)
-        set_pos                         // float (absolute position)
+        set_pos,                        // float (absolute position)
+        set_loop_mode,                  // parameters::loop_mode_t
+        set_fullscreen_screen,          // int
     };
     
     type type;
@@ -146,7 +148,9 @@ public:
         subtitle_scale,         // float
         subtitle_color,         // uint64_t
         subtitle_parallax,      // float
-        pos                     // float
+        pos,                    // float
+        loop_mode,              // parameters::loop_mode_t
+        fullscreen_screen,      // int
     };
     
     type type;
