@@ -147,8 +147,20 @@ int main(int argc, char *argv[])
     log_levels.push_back("warning");
     log_levels.push_back("error");
     log_levels.push_back("quiet");
-    opt::val<std::string> log_level("log-level", 'l', opt::optional, log_levels, "");
+    opt::val<std::string> log_level("log-level", 'L', opt::optional, log_levels, "");
     options.push_back(&log_level);
+    std::vector<std::string> device_types;
+    device_types.push_back("default");
+    device_types.push_back("firewire");
+    device_types.push_back("x11");
+    opt::val<std::string> device_type("device-type", '\0', opt::optional, device_types, "");
+    options.push_back(&device_type);
+    opt::tuple<int> device_frame_size("device-frame-size", '\0', opt::optional,
+            1, std::numeric_limits<int>::max(), std::vector<int>(2, 0), 2, "x");
+    options.push_back(&device_frame_size);
+    opt::tuple<int> device_frame_rate("device-frame-rate", '\0', opt::optional,
+            1, std::numeric_limits<int>::max(), std::vector<int>(2, 0), 2, "/");
+    options.push_back(&device_frame_rate);
     std::vector<std::string> input_modes;
     input_modes.push_back("mono");
     input_modes.push_back("separate-left-right");
@@ -205,6 +217,8 @@ int main(int argc, char *argv[])
     options.push_back(&swap_eyes);
     opt::flag fullscreen("fullscreen", 'f', opt::optional);
     options.push_back(&fullscreen);
+    opt::val<int> fullscreen_screen("fullscreen-screen", '\0', opt::optional, 0, 999, parameters().fullscreen_screen);
+    options.push_back(&fullscreen_screen);
     opt::flag center("center", 'c', opt::optional);
     options.push_back(&center);
     opt::val<std::string> subtitle_encoding("subtitle-encoding", '\0', opt::optional, parameters().subtitle_encoding);
@@ -228,6 +242,8 @@ int main(int argc, char *argv[])
     options.push_back(&ghostbust);
     opt::flag benchmark("benchmark", 'b', opt::optional);
     options.push_back(&benchmark);
+    opt::flag loop("loop", 'l', opt::optional);
+    options.push_back(&loop);
     // Accept some Equalizer options. These are passed to Equalizer for interpretation.
     opt::val<std::string> eq_server("eq-server", '\0', opt::optional);
     options.push_back(&eq_server);
@@ -282,7 +298,10 @@ int main(int argc, char *argv[])
                     "  --help                   Print help.\n"
                     "  --version                Print version.\n"
                     "  -n|--no-gui              Do not use the GUI, just show a plain window.\n"
-                    "  -l|--log-level=LEVEL     Set log level (debug/info/warning/error/quiet).\n"
+                    "  -L|--log-level=LEVEL     Set log level (debug/info/warning/error/quiet).\n"
+                    "  --device-type=TYPE       Type of input device: default, firewire, or x11.\n"
+                    "  --device-frame-size=WxH  Request frame size WxH from input device.\n"
+                    "  --device-frame-rate=N/D  Request frame rate N/D from input device.\n"
                     "  -v|--video=STREAM        Select video stream (1-n, depending on input).\n"
                     "  -a|--audio=STREAM        Select audio stream (1-n, depending on input).\n"
                     "  -s|--subtitle=STREAM     Select subtitle stream (0-n, depending on input).\n"
@@ -330,6 +349,8 @@ int main(int argc, char *argv[])
                     "    equalizer-3d             Multi-display via Equalizer (3D setup).\n"
                     "  -S|--swap-eyes           Swap left/right view.\n"
                     "  -f|--fullscreen          Fullscreen.\n"
+                    "  --fullscreen-screen=N    Use screen N for fullscreen mode. The default\n"
+                    "                           N=0 uses the primary screen of the desktop.\n"
                     "  -c|--center              Center window on screen.\n"
                     "  --subtitle-encoding=ENC  Set subtitle encoding.\n"
                     "  --subtitle-font=FONT     Set subtitle font name.\n"
@@ -342,6 +363,7 @@ int main(int argc, char *argv[])
                     "                           values for the R,G,B channels.\n"
                     "  -G|--ghostbust=VAL       Amount of ghostbusting to apply (0 to 1).\n"
                     "  -b|--benchmark           Benchmark mode (no audio, show fps).\n"
+                    "  -l|--loop                Loop the input media.\n"
                     "\n"
                     "Interactive control:\n"
                     "  ESC                      Leave fullscreen mode, or quit.\n"
@@ -415,6 +437,23 @@ int main(int argc, char *argv[])
     {
         init_data.log_level = msg::REQ;
     }
+    if (device_type.value() == "")
+    {
+        init_data.dev_request.device =
+            (arguments.size() == 1 && arguments[0].substr(0, 5) == "/dev/"
+             ? device_request::sys_default : device_request::no_device);
+    }
+    else
+    {
+        init_data.dev_request.device =
+            (device_type.value() == "firewire" ? device_request::firewire
+             : device_type.value() == "x11" ? device_request::x11
+             : device_request::sys_default);
+    }
+    init_data.dev_request.width = device_frame_size.value()[0];
+    init_data.dev_request.height = device_frame_size.value()[1];
+    init_data.dev_request.frame_rate_num = device_frame_rate.value()[0];
+    init_data.dev_request.frame_rate_den = device_frame_rate.value()[1];
     init_data.urls = arguments;
     init_data.video_stream = video.value() - 1;
     init_data.audio_stream = audio.value() - 1;
@@ -456,6 +495,7 @@ int main(int argc, char *argv[])
     }
     init_data.params.stereo_mode_swap = swap_eyes.value();
     init_data.fullscreen = fullscreen.value();
+    init_data.params.fullscreen_screen = fullscreen_screen.value();
     init_data.center = center.value();
     init_data.params.subtitle_encoding = subtitle_encoding.value();
     init_data.params.subtitle_font = subtitle_font.value();
@@ -476,6 +516,7 @@ int main(int argc, char *argv[])
     {
         msg::inf(_("Benchmark mode: audio and time synchronization disabled."));
     }
+    init_data.params.loop_mode = (loop.value() ? parameters::loop_current : parameters::no_loop);
 
     int retval = 0;
     player *player = NULL;
