@@ -35,6 +35,36 @@
 #include "dbg.h"
 
 
+device_request::device_request() :
+    device(no_device),
+    width(0),
+    height(0),
+    frame_rate_num(0),
+    frame_rate_den(0)
+{
+}
+
+void device_request::save(std::ostream &os) const
+{
+    s11n::save(os, static_cast<int>(device));
+    s11n::save(os, width);
+    s11n::save(os, height);
+    s11n::save(os, frame_rate_num);
+    s11n::save(os, frame_rate_den);
+}
+
+void device_request::load(std::istream &is)
+{
+    int x;
+    s11n::load(is, x);
+    device = static_cast<device_t>(x);
+    s11n::load(is, width);
+    s11n::load(is, height);
+    s11n::load(is, frame_rate_num);
+    s11n::load(is, frame_rate_den);
+}
+
+
 video_frame::video_frame() :
     raw_width(-1),
     raw_height(-1),
@@ -540,7 +570,9 @@ parameters::parameters() :
     subtitle_size(std::numeric_limits<int>::min()),
     subtitle_scale(-1.0f),
     subtitle_color(std::numeric_limits<uint64_t>::max()),
-    subtitle_parallax(std::numeric_limits<float>::quiet_NaN())
+    subtitle_parallax(std::numeric_limits<float>::quiet_NaN()),
+    loop_mode(no_loop),
+    fullscreen_screen(-1)
 {
 }
 
@@ -606,6 +638,10 @@ void parameters::set_defaults()
     {
         subtitle_parallax = 0.0f;
     }
+    if (fullscreen_screen < 0)
+    {
+        fullscreen_screen = 0;
+    }
 }
 
 std::string parameters::stereo_mode_to_string(stereo_mode_t stereo_mode, bool stereo_mode_swap)
@@ -836,6 +872,8 @@ void parameters::save(std::ostream &os) const
     s11n::save(os, subtitle_scale);
     s11n::save(os, subtitle_color);
     s11n::save(os, subtitle_parallax);
+    s11n::save(os, static_cast<int>(loop_mode));
+    s11n::save(os, fullscreen_screen);
 }
 
 void parameters::load(std::istream &is)
@@ -859,4 +897,7 @@ void parameters::load(std::istream &is)
     s11n::load(is, subtitle_scale);
     s11n::load(is, subtitle_color);
     s11n::load(is, subtitle_parallax);
+    s11n::load(is, x);
+    loop_mode = static_cast<loop_mode_t>(x);
+    s11n::load(is, fullscreen_screen);
 }
