@@ -28,6 +28,37 @@
 #include "s11n.h"
 
 
+class device_request : public s11n
+{
+public:
+    typedef enum
+    {
+        no_device,      // No device request.
+        sys_default,    // Request for system default video device type.
+        firewire,       // Request for a firefire video device.
+        x11,            // Request for an X11 grabber.
+    } device_t;
+
+    device_t device;    // The device type.
+    int width;          // Request frames of the given width (0 means default).
+    int height;         // Request frames of the given height (0 means default).
+    int frame_rate_num; // Request a specific frame rate (0/0 means default).
+    int frame_rate_den; // For example 1/25, 1/30, ...
+
+    // Constructor
+    device_request();
+
+    // Is this a request for a device?
+    bool is_device() const
+    {
+        return device != no_device;
+    }
+
+    // Serialization
+    void save(std::ostream &os) const;
+    void load(std::istream &is);
+};
+
 class video_frame
 {
 public:
@@ -270,6 +301,12 @@ public:
         red_blue_monochrome,            // Red/blue anaglyph, monochrome method
     } stereo_mode_t;
 
+    typedef enum
+    {
+        no_loop,                        // Do not loop.
+        loop_current,                   // Loop the current media input.
+    } loop_mode_t;
+
     stereo_mode_t stereo_mode;          // Stereo mode
     bool stereo_mode_swap;              // Swap left and right view
     float parallax;                     // Parallax adjustment, -1 .. +1
@@ -287,6 +324,8 @@ public:
     float subtitle_scale;               // Scale factor
     uint64_t subtitle_color;            // Subtitle color in uint32_t bgra32 format, > UINT32_MAX means keep default
     float subtitle_parallax;            // Subtitle parallax adjustment, -1 .. +1
+    loop_mode_t loop_mode;              // Current loop behaviour.
+    int fullscreen_screen;              // Screen to use in fullscreen mode: 0 (primary screen), 1, 2, ...
 
     // Constructor
     parameters();
