@@ -26,6 +26,7 @@ extern "C"
 {
 #define __STDC_CONSTANT_MACROS
 #include <libavformat/avformat.h>
+#include <libavdevice/avdevice.h>
 #include <libavcodec/avcodec.h>
 #include <libswscale/swscale.h>
 }
@@ -62,11 +63,12 @@ class read_thread : public thread
 {
 private:
     const std::string _url;
+    const bool _is_device;
     struct ffmpeg_stuff *_ffmpeg;
     bool _eof;
 
 public:
-    read_thread(const std::string &url, struct ffmpeg_stuff *ffmpeg);
+    read_thread(const std::string &url, bool is_device, struct ffmpeg_stuff *ffmpeg);
     void run();
     void reset();
     bool eof() const
@@ -315,6 +317,7 @@ static int64_t stream_duration(AVStream *stream, AVFormatContext *format)
 
 media_object::media_object() : _ffmpeg(NULL)
 {
+    avdevice_register_all();
     av_register_all();
     switch (msg::level())
     {
@@ -628,20 +631,95 @@ void media_object::set_subtitle_box_template(int index)
     }
 }
 
-void media_object::open(const std::string &url)
+void media_object::open(const std::string &url, const device_request &dev_request)
 {
     assert(!_ffmpeg);
 
     _url = url;
+    _is_device = dev_request.is_device();
     _ffmpeg = new struct ffmpeg_stuff;
-    _ffmpeg->reader = new read_thread(_url, _ffmpeg);
+    _ffmpeg->reader = new read_thread(_url, _is_device, _ffmpeg);
     int e;
 
-    if ((e = av_open_input_file(&_ffmpeg->format_ctx, _url.c_str(), NULL, 0, NULL)) != 0)
+    /* Set format and parameters for device input */
+    AVInputFormat *iformat;
+    AVFormatParameters iparams;
+    bool use_iparams = false;
+    switch (dev_request.device)
+    {
+    case device_request::firewire:
+        iformat = av_find_input_format("libdc1394");
+        break;
+    case device_request::x11:
+        iformat = av_find_input_format("x11grab");
+        break;
+    case device_request::sys_default:
+#if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
+        iformat = av_find_input_format("vfwcap");
+        // vfwcap requires a time_base parameter. Set the default to 1/25.
+        std::memset(&iparams, 0, sizeof(iparams));
+        iparams.time_base.num = 1;
+        iparams.time_base.den = 25;
+        use_iparams = true;
+#elif defined __FreeBSD__ || defined __NetBSD__ || defined __OpenBSD__ || defined __APPLE__
+        iformat = av_find_input_format("bktr");
+        // bktr requires width, height, time_base parameters.
+        // Set the defaults to 640x480 and 1/25.
+        std::memset(&iparams, 0, sizeof(iparams));
+        iparams.width = 640;
+        iparams.height = 480;
+        iparams.time_base.num = 1;
+        iparams.time_base.den = 25;
+        use_iparams = true;
+#else
+        iformat = av_find_input_format("video4linux2");
+        // video4linux2 can use default parameters.
+#endif
+        break;
+    case device_request::no_device:
+        iformat = NULL;
+        break;
+    }
+    if (_is_device && !iformat)
+    {
+        throw exc(str::asprintf(_("No support available for %s device."),
+                dev_request.device == device_request::firewire ? _("Firewire")
+                : dev_request.device == device_request::x11 ? _("X11")
+                : _("default")));
+    }
+    if (_is_device
+            && ((dev_request.width != 0 && dev_request.height != 0)
+                || (dev_request.frame_rate_num != 0 && dev_request.frame_rate_den != 0)))
+    {
+        if (!use_iparams)
+        {
+            std::memset(&iparams, 0, sizeof(iparams));
+            use_iparams = true;
+        }
+        if (dev_request.width != 0 && dev_request.height != 0)
+        {
+            iparams.width = dev_request.width;
+            iparams.height = dev_request.height;
+        }
+        if (dev_request.frame_rate_num != 0 && dev_request.frame_rate_den != 0)
+        {
+            iparams.time_base.den = dev_request.frame_rate_num;
+            iparams.time_base.num = dev_request.frame_rate_den;
+        }
+    }
+
+    /* Open the input */
+    if ((e = av_open_input_file(&_ffmpeg->format_ctx, _url.c_str(), iformat, 0,
+                    use_iparams ? &iparams : NULL)) != 0)
     {
         throw exc(str::asprintf(_("%s: %s"),
                     _url.c_str(), my_av_strerror(e).c_str()));
     }
+    if (_is_device)
+    {
+        // For a camera device, do not read ahead multiple packets, to avoid a startup delay.
+        _ffmpeg->format_ctx->max_analyze_duration = 0;
+    }
     if ((e = av_find_stream_info(_ffmpeg->format_ctx)) < 0)
     {
         throw exc(str::asprintf(_("%s: Cannot read stream info: %s"),
@@ -1037,8 +1115,8 @@ int64_t media_object::subtitle_duration(int index) const
             _ffmpeg->format_ctx);
 }
 
-read_thread::read_thread(const std::string &url, struct ffmpeg_stuff *ffmpeg) :
-    _url(url), _ffmpeg(ffmpeg), _eof(false)
+read_thread::read_thread(const std::string &url, bool is_device, struct ffmpeg_stuff *ffmpeg) :
+    _url(url), _is_device(is_device), _ffmpeg(ffmpeg), _eof(false)
 {
 }
 
@@ -1047,9 +1125,11 @@ void read_thread::run()
     while (!_eof)
     {
         // We need another packet if the number of queued packets for an active stream is below a threshold.
-        const size_t video_stream_low_threshold = 2;    // Often, 1 packet results in one video frame
-        const size_t audio_stream_low_threshold = 5;    // Often, 3-4 packets are needed for one buffer fill
-        const size_t subtitle_stream_low_threshold = 1; // Just a guess
+        // For files, we often want to read ahead to avoid i/o waits. For devices, we do not want to read
+        // ahead to avoid latency.
+        const size_t video_stream_low_threshold = (_is_device ? 1 : 2);         // Often, 1 packet results in one video frame
+        const size_t audio_stream_low_threshold = (_is_device ? 1 : 5);         // Often, 3-4 packets are needed for one buffer fill
+        const size_t subtitle_stream_low_threshold = (_is_device ? 1 : 1);      // Just a guess
         bool need_another_packet = false;
         for (size_t i = 0; !need_another_packet && i < _ffmpeg->video_streams.size(); i++)
         {
@@ -1065,8 +1145,8 @@ void read_thread::run()
             if (_ffmpeg->format_ctx->streams[_ffmpeg->audio_streams[i]]->discard == AVDISCARD_DEFAULT)
             {
                 _ffmpeg->audio_packet_queue_mutexes[i].lock();
-                need_another_packet = _ffmpeg->audio_packet_queues[i].size() < audio_stream_low_threshold;
-                _ffmpeg->audio_packet_queue_mutexes[i].unlock();
+                    need_another_packet = _ffmpeg->audio_packet_queues[i].size() < audio_stream_low_threshold;
+                    _ffmpeg->audio_packet_queue_mutexes[i].unlock();
             }
         }
         for (size_t i = 0; !need_another_packet && i < _ffmpeg->subtitle_streams.size(); i++)
@@ -1702,113 +1782,120 @@ void media_object::seek(int64_t dest_pos)
 
 void media_object::close()
 {
-    try
+    if (_ffmpeg)
     {
-        // Stop decoder threads
-        for (size_t i = 0; i < _ffmpeg->video_decode_threads.size(); i++)
+        try
         {
-            _ffmpeg->video_decode_threads[i].finish();
+            // Stop decoder threads
+            for (size_t i = 0; i < _ffmpeg->video_decode_threads.size(); i++)
+            {
+                _ffmpeg->video_decode_threads[i].finish();
+            }
+            for (size_t i = 0; i < _ffmpeg->audio_decode_threads.size(); i++)
+            {
+                _ffmpeg->audio_decode_threads[i].finish();
+            }
+            for (size_t i = 0; i < _ffmpeg->subtitle_decode_threads.size(); i++)
+            {
+                _ffmpeg->subtitle_decode_threads[i].finish();
+            }
+            // Stop reading packets
+            _ffmpeg->reader->finish();
         }
-        for (size_t i = 0; i < _ffmpeg->audio_decode_threads.size(); i++)
+        catch (...)
         {
-            _ffmpeg->audio_decode_threads[i].finish();
         }
-        for (size_t i = 0; i < _ffmpeg->subtitle_decode_threads.size(); i++)
+        if (_ffmpeg->format_ctx)
         {
-            _ffmpeg->subtitle_decode_threads[i].finish();
+            for (size_t i = 0; i < _ffmpeg->video_frames.size(); i++)
+            {
+                av_free(_ffmpeg->video_frames[i]);
+            }
+            for (size_t i = 0; i < _ffmpeg->video_out_frames.size(); i++)
+            {
+                av_free(_ffmpeg->video_out_frames[i]);
+            }
+            for (size_t i = 0; i < _ffmpeg->video_buffers.size(); i++)
+            {
+                av_free(_ffmpeg->video_buffers[i]);
+            }
+            for (size_t i = 0; i < _ffmpeg->video_codec_ctxs.size(); i++)
+            {
+                if (i < _ffmpeg->video_codecs.size() && _ffmpeg->video_codecs[i])
+                {
+                    avcodec_close(_ffmpeg->video_codec_ctxs[i]);
+                }
+            }
+            for (size_t i = 0; i < _ffmpeg->video_img_conv_ctxs.size(); i++)
+            {
+                sws_freeContext(_ffmpeg->video_img_conv_ctxs[i]);
+            }
+            for (size_t i = 0; i < _ffmpeg->video_packet_queues.size(); i++)
+            {
+                if (_ffmpeg->video_packet_queues[i].size() > 0)
+                {
+                    msg::dbg(_url + ": " + str::from(_ffmpeg->video_packet_queues[i].size())
+                            + " unprocessed packets in video stream " + str::from(i));
+                }
+                for (size_t j = 0; j < _ffmpeg->video_packet_queues[i].size(); j++)
+                {
+                    av_free_packet(&_ffmpeg->video_packet_queues[i][j]);
+                }
+            }
+            for (size_t i = 0; i < _ffmpeg->video_packets.size(); i++)
+            {
+                av_free_packet(&(_ffmpeg->video_packets[i]));
+            }
+            for (size_t i = 0; i < _ffmpeg->audio_codec_ctxs.size(); i++)
+            {
+                if (i < _ffmpeg->audio_codecs.size() && _ffmpeg->audio_codecs[i])
+                {
+                    avcodec_close(_ffmpeg->audio_codec_ctxs[i]);
+                }
+            }
+            for (size_t i = 0; i < _ffmpeg->audio_packet_queues.size(); i++)
+            {
+                if (_ffmpeg->audio_packet_queues[i].size() > 0)
+                {
+                    msg::dbg(_url + ": " + str::from(_ffmpeg->audio_packet_queues[i].size())
+                            + " unprocessed packets in audio stream " + str::from(i));
+                }
+                for (size_t j = 0; j < _ffmpeg->audio_packet_queues[i].size(); j++)
+                {
+                    av_free_packet(&_ffmpeg->audio_packet_queues[i][j]);
+                }
+            }
+            for (size_t i = 0; i < _ffmpeg->audio_tmpbufs.size(); i++)
+            {
+                av_free(_ffmpeg->audio_tmpbufs[i]);
+            }
+            for (size_t i = 0; i < _ffmpeg->subtitle_codec_ctxs.size(); i++)
+            {
+                if (i < _ffmpeg->subtitle_codecs.size() && _ffmpeg->subtitle_codecs[i])
+                {
+                    avcodec_close(_ffmpeg->subtitle_codec_ctxs[i]);
+                }
+            }
+            for (size_t i = 0; i < _ffmpeg->subtitle_packet_queues.size(); i++)
+            {
+                if (_ffmpeg->subtitle_packet_queues[i].size() > 0)
+                {
+                    msg::dbg(_url + ": " + str::from(_ffmpeg->subtitle_packet_queues[i].size())
+                            + " unprocessed packets in subtitle stream " + str::from(i));
+                }
+                for (size_t j = 0; j < _ffmpeg->subtitle_packet_queues[i].size(); j++)
+                {
+                    av_free_packet(&_ffmpeg->subtitle_packet_queues[i][j]);
+                }
+            }
+            av_close_input_file(_ffmpeg->format_ctx);
         }
-        // Stop reading packets
-        _ffmpeg->reader->finish();
+        delete _ffmpeg->reader;
+        delete _ffmpeg;
+        _ffmpeg = NULL;
     }
-    catch (...)
-    {
-    }
-    for (size_t i = 0; i < _ffmpeg->video_frames.size(); i++)
-    {
-        av_free(_ffmpeg->video_frames[i]);
-    }
-    for (size_t i = 0; i < _ffmpeg->video_out_frames.size(); i++)
-    {
-        av_free(_ffmpeg->video_out_frames[i]);
-    }
-    for (size_t i = 0; i < _ffmpeg->video_buffers.size(); i++)
-    {
-        av_free(_ffmpeg->video_buffers[i]);
-    }
-    for (size_t i = 0; i < _ffmpeg->video_codec_ctxs.size(); i++)
-    {
-        if (i < _ffmpeg->video_codecs.size() && _ffmpeg->video_codecs[i])
-        {
-            avcodec_close(_ffmpeg->video_codec_ctxs[i]);
-        }
-    }
-    for (size_t i = 0; i < _ffmpeg->video_img_conv_ctxs.size(); i++)
-    {
-        sws_freeContext(_ffmpeg->video_img_conv_ctxs[i]);
-    }
-    for (size_t i = 0; i < _ffmpeg->video_packet_queues.size(); i++)
-    {
-        if (_ffmpeg->video_packet_queues[i].size() > 0)
-        {
-            msg::dbg(_url + ": " + str::from(_ffmpeg->video_packet_queues[i].size())
-                    + " unprocessed packets in video stream " + str::from(i));
-        }
-        for (size_t j = 0; j < _ffmpeg->video_packet_queues[i].size(); j++)
-        {
-            av_free_packet(&_ffmpeg->video_packet_queues[i][j]);
-        }
-    }
-    for (size_t i = 0; i < _ffmpeg->video_packets.size(); i++)
-    {
-        av_free_packet(&(_ffmpeg->video_packets[i]));
-    }
-    for (size_t i = 0; i < _ffmpeg->audio_codec_ctxs.size(); i++)
-    {
-        if (i < _ffmpeg->audio_codecs.size() && _ffmpeg->audio_codecs[i])
-        {
-            avcodec_close(_ffmpeg->audio_codec_ctxs[i]);
-        }
-    }
-    for (size_t i = 0; i < _ffmpeg->audio_packet_queues.size(); i++)
-    {
-        if (_ffmpeg->audio_packet_queues[i].size() > 0)
-        {
-            msg::dbg(_url + ": " + str::from(_ffmpeg->audio_packet_queues[i].size())
-                    + " unprocessed packets in audio stream " + str::from(i));
-        }
-        for (size_t j = 0; j < _ffmpeg->audio_packet_queues[i].size(); j++)
-        {
-            av_free_packet(&_ffmpeg->audio_packet_queues[i][j]);
-        }
-    }
-    for (size_t i = 0; i < _ffmpeg->audio_tmpbufs.size(); i++)
-    {
-        av_free(_ffmpeg->audio_tmpbufs[i]);
-    }
-    for (size_t i = 0; i < _ffmpeg->subtitle_codec_ctxs.size(); i++)
-    {
-        if (i < _ffmpeg->subtitle_codecs.size() && _ffmpeg->subtitle_codecs[i])
-        {
-            avcodec_close(_ffmpeg->subtitle_codec_ctxs[i]);
-        }
-    }
-    for (size_t i = 0; i < _ffmpeg->subtitle_packet_queues.size(); i++)
-    {
-        if (_ffmpeg->subtitle_packet_queues[i].size() > 0)
-        {
-            msg::dbg(_url + ": " + str::from(_ffmpeg->subtitle_packet_queues[i].size())
-                    + " unprocessed packets in subtitle stream " + str::from(i));
-        }
-        for (size_t j = 0; j < _ffmpeg->subtitle_packet_queues[i].size(); j++)
-        {
-            av_free_packet(&_ffmpeg->subtitle_packet_queues[i][j]);
-        }
-    }
-    if (_ffmpeg->format_ctx)
-    {
-        av_close_input_file(_ffmpeg->format_ctx);
-    }
-    delete _ffmpeg->reader;
-    delete _ffmpeg;
-    _ffmpeg = NULL;
+    _url = "";
+    _is_device = false;
+    _tag_names.clear();
+    _tag_values.clear();
 }
