@@ -43,6 +43,7 @@
 
 player_init_data::player_init_data() :
     log_level(msg::INF),
+    dev_request(),
     urls(),
     video_stream(0),
     audio_stream(0),
@@ -67,6 +68,7 @@ player_init_data::~player_init_data()
 void player_init_data::save(std::ostream &os) const
 {
     s11n::save(os, static_cast<int>(log_level));
+    s11n::save(os, dev_request);
     s11n::save(os, urls);
     s11n::save(os, video_stream);
     s11n::save(os, audio_stream);
@@ -88,6 +90,7 @@ void player_init_data::load(std::istream &is)
     int x;
     s11n::load(is, x);
     log_level = static_cast<msg::level_t>(x);
+    s11n::load(is, dev_request);
     s11n::load(is, urls);
     s11n::load(is, video_stream);
     s11n::load(is, audio_stream);
@@ -200,7 +203,7 @@ void player::open(const player_init_data &init_data)
 
     // Create media input
     _media_input = new media_input();
-    _media_input->open(init_data.urls);
+    _media_input->open(init_data.urls, init_data.dev_request);
     if (_media_input->video_streams() == 0)
     {
         throw exc(_("No video streams found."));
@@ -288,7 +291,7 @@ void player::open(const player_init_data &init_data)
                 _params.stereo_mode);
         if (init_data.fullscreen)
         {
-            _video_output->enter_fullscreen();
+            _video_output->enter_fullscreen(_params.fullscreen_screen);
         }
         if (init_data.center)
         {
@@ -516,7 +519,15 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             else
             {
                 msg::dbg("End of video stream.");
-                stop_playback();
+                if (_params.loop_mode == parameters::loop_current)
+                {
+                    _set_pos_request = 0.0f;
+                    *more_steps = true;
+                }
+                else
+                {
+                    stop_playback();
+                }
                 return 0;
             }
         }
@@ -592,7 +603,15 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
                 if (!blob.is_valid())
                 {
                     msg::dbg("End of audio stream.");
-                    stop_playback();
+                    if (_params.loop_mode == parameters::loop_current)
+                    {
+                        _set_pos_request = 0.0f;
+                        *more_steps = true;
+                    }
+                    else
+                    {
+                        stop_playback();
+                    }
                     return 0;
                 }
                 _audio_pos = blob.presentation_time;
@@ -612,11 +631,12 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         }
 
         int64_t allowable_sleep = 0;
-        if (_master_time_current >= _video_pos || _benchmark)
+        if (_master_time_current >= _video_pos || _benchmark || _media_input->is_device())
         {
             // Output current video frame
             _drop_next_frame = false;
-            if (_master_time_current - _video_pos > _media_input->video_frame_duration() * 75 / 100 && !_benchmark)
+            if (_master_time_current - _video_pos > _media_input->video_frame_duration() * 75 / 100
+                    && !_benchmark && !_media_input->is_device())
             {
                 msg::wrn(_("Video: delay %g seconds; dropping next frame."), (_master_time_current - _video_pos) / 1e6f);
                 _drop_next_frame = true;
@@ -868,7 +888,7 @@ void player::receive_cmd(const command &cmd)
         flag = false;
         if (_video_output)
         {
-            flag = _video_output->toggle_fullscreen();
+            flag = _video_output->toggle_fullscreen(_params.fullscreen_screen);
         }
         notify(notification::fullscreen, flag, !flag);
         break;
@@ -1050,6 +1070,23 @@ void player::receive_cmd(const command &cmd)
         _set_pos_request = param;
         /* notify when request is fulfilled */
         break;
+    case command::set_loop_mode:
+        {
+            int old_loop_mode = static_cast<int>(_params.loop_mode);
+            int loop_mode;
+            s11n::load(p, loop_mode);
+            _params.loop_mode = static_cast<parameters::loop_mode_t>(loop_mode);
+            parameters_changed = true;
+            notify(notification::loop_mode, old_loop_mode, loop_mode);
+        }
+        break;
+    case command::set_fullscreen_screen:
+        {
+            int old_screen = _params.fullscreen_screen;
+            s11n::load(p, _params.fullscreen_screen);
+            notify(notification::fullscreen_screen, old_screen, _params.fullscreen_screen);
+            break;
+        }
     }
 
     if (parameters_changed && _video_output)
